"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .basesdk import BaseSDK
from ascend_sdk import utils
from ascend_sdk._hooks import HookContext
from ascend_sdk.models import components, errors, operations
from ascend_sdk.types import OptionalNullable, UNSET
from typing import Any, Optional, Union


class TradeAllocation(BaseSDK):
    def create_trade_allocation(
        self,
        *,
        account_id: str,
        trade_allocation_create: Union[
            components.TradeAllocationCreate, components.TradeAllocationCreateTypedDict
        ],
        request_id: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.BookingCreateTradeAllocationResponse:
        r"""Create Trade Allocation

        Creates a new trade allocation. These are used to allocate or distribute positions between Apex accounts.

        Upon success, returns the created trade allocation and its enriched details.

        :param account_id: The account id.
        :param trade_allocation_create:
        :param request_id: A globally unique UUID that is specific to the request. This id is used to prevent duplicate requests from being processed.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = operations.BookingCreateTradeAllocationRequest(
            account_id=account_id,
            request_id=request_id,
            trade_allocation_create=utils.get_pydantic_model(
                trade_allocation_create, components.TradeAllocationCreate
            ),
        )

        req = self.build_request(
            method="POST",
            path="/booking/v1/accounts/{account_id}/tradeAllocations",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.trade_allocation_create,
                False,
                False,
                "json",
                components.TradeAllocationCreate,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="Booking_CreateTradeAllocation",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "403",
                "404",
                "409",
                "4XX",
                "500",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.BookingCreateTradeAllocationResponse(
                trade_allocation=utils.unmarshal_json(
                    http_res.text, Optional[components.TradeAllocation]
                ),
                http_meta=components.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(
            http_res,
            ["400", "403", "404", "409", "500", "503", "504"],
            "application/json",
        ):
            data = utils.unmarshal_json(http_res.text, errors.StatusData)
            raise errors.Status(data=data)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )
        if utils.match_response(http_res, "default", "application/json"):
            return operations.BookingCreateTradeAllocationResponse(
                status=utils.unmarshal_json(http_res.text, Optional[components.Status]),
                http_meta=components.HTTPMetadata(request=req, response=http_res),
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    async def create_trade_allocation_async(
        self,
        *,
        account_id: str,
        trade_allocation_create: Union[
            components.TradeAllocationCreate, components.TradeAllocationCreateTypedDict
        ],
        request_id: Optional[str] = None,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.BookingCreateTradeAllocationResponse:
        r"""Create Trade Allocation

        Creates a new trade allocation. These are used to allocate or distribute positions between Apex accounts.

        Upon success, returns the created trade allocation and its enriched details.

        :param account_id: The account id.
        :param trade_allocation_create:
        :param request_id: A globally unique UUID that is specific to the request. This id is used to prevent duplicate requests from being processed.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = operations.BookingCreateTradeAllocationRequest(
            account_id=account_id,
            request_id=request_id,
            trade_allocation_create=utils.get_pydantic_model(
                trade_allocation_create, components.TradeAllocationCreate
            ),
        )

        req = self.build_request_async(
            method="POST",
            path="/booking/v1/accounts/{account_id}/tradeAllocations",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.trade_allocation_create,
                False,
                False,
                "json",
                components.TradeAllocationCreate,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="Booking_CreateTradeAllocation",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=[
                "400",
                "403",
                "404",
                "409",
                "4XX",
                "500",
                "503",
                "504",
                "5XX",
            ],
            retry_config=retry_config,
        )

        data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.BookingCreateTradeAllocationResponse(
                trade_allocation=utils.unmarshal_json(
                    http_res.text, Optional[components.TradeAllocation]
                ),
                http_meta=components.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(
            http_res,
            ["400", "403", "404", "409", "500", "503", "504"],
            "application/json",
        ):
            data = utils.unmarshal_json(http_res.text, errors.StatusData)
            raise errors.Status(data=data)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )
        if utils.match_response(http_res, "default", "application/json"):
            return operations.BookingCreateTradeAllocationResponse(
                status=utils.unmarshal_json(http_res.text, Optional[components.Status]),
                http_meta=components.HTTPMetadata(request=req, response=http_res),
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    def get_trade_allocation(
        self,
        *,
        account_id: str,
        trade_allocation_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.BookingGetTradeAllocationResponse:
        r"""Get Trade Allocation

        Retrieves a trade allocation and its details.

        Upon successful submission, returns the trade allocation details.

        :param account_id: The account id.
        :param trade_allocation_id: The tradeAllocation id.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = operations.BookingGetTradeAllocationRequest(
            account_id=account_id,
            trade_allocation_id=trade_allocation_id,
        )

        req = self.build_request(
            method="GET",
            path="/booking/v1/accounts/{account_id}/tradeAllocations/{tradeAllocation_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="Booking_GetTradeAllocation",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "403", "404", "4XX", "500", "503", "504", "5XX"],
            retry_config=retry_config,
        )

        data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.BookingGetTradeAllocationResponse(
                trade_allocation=utils.unmarshal_json(
                    http_res.text, Optional[components.TradeAllocation]
                ),
                http_meta=components.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(
            http_res, ["400", "403", "404", "500", "503", "504"], "application/json"
        ):
            data = utils.unmarshal_json(http_res.text, errors.StatusData)
            raise errors.Status(data=data)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )
        if utils.match_response(http_res, "default", "application/json"):
            return operations.BookingGetTradeAllocationResponse(
                status=utils.unmarshal_json(http_res.text, Optional[components.Status]),
                http_meta=components.HTTPMetadata(request=req, response=http_res),
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    async def get_trade_allocation_async(
        self,
        *,
        account_id: str,
        trade_allocation_id: str,
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.BookingGetTradeAllocationResponse:
        r"""Get Trade Allocation

        Retrieves a trade allocation and its details.

        Upon successful submission, returns the trade allocation details.

        :param account_id: The account id.
        :param trade_allocation_id: The tradeAllocation id.
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = operations.BookingGetTradeAllocationRequest(
            account_id=account_id,
            trade_allocation_id=trade_allocation_id,
        )

        req = self.build_request_async(
            method="GET",
            path="/booking/v1/accounts/{account_id}/tradeAllocations/{tradeAllocation_id}",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=False,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="Booking_GetTradeAllocation",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "403", "404", "4XX", "500", "503", "504", "5XX"],
            retry_config=retry_config,
        )

        data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.BookingGetTradeAllocationResponse(
                trade_allocation=utils.unmarshal_json(
                    http_res.text, Optional[components.TradeAllocation]
                ),
                http_meta=components.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(
            http_res, ["400", "403", "404", "500", "503", "504"], "application/json"
        ):
            data = utils.unmarshal_json(http_res.text, errors.StatusData)
            raise errors.Status(data=data)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )
        if utils.match_response(http_res, "default", "application/json"):
            return operations.BookingGetTradeAllocationResponse(
                status=utils.unmarshal_json(http_res.text, Optional[components.Status]),
                http_meta=components.HTTPMetadata(request=req, response=http_res),
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    def cancel_trade_allocation(
        self,
        *,
        account_id: str,
        trade_allocation_id: str,
        cancel_trade_allocation_request_create: Union[
            components.CancelTradeAllocationRequestCreate,
            components.CancelTradeAllocationRequestCreateTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.BookingCancelTradeAllocationResponse:
        r"""Cancel Trade Allocation

        Cancel a trade allocation using the original trade_allocation_id.

        Upon successful submission, returns an empty response. CancelTradeAllocation will either cancel everything, or nothing at all if a failure occurs.

        :param account_id: The account id.
        :param trade_allocation_id: The tradeAllocation id.
        :param cancel_trade_allocation_request_create:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = operations.BookingCancelTradeAllocationRequest(
            account_id=account_id,
            trade_allocation_id=trade_allocation_id,
            cancel_trade_allocation_request_create=utils.get_pydantic_model(
                cancel_trade_allocation_request_create,
                components.CancelTradeAllocationRequestCreate,
            ),
        )

        req = self.build_request(
            method="POST",
            path="/booking/v1/accounts/{account_id}/tradeAllocations/{tradeAllocation_id}:cancel",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.cancel_trade_allocation_request_create,
                False,
                False,
                "json",
                components.CancelTradeAllocationRequestCreate,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="Booking_CancelTradeAllocation",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "403", "404", "4XX", "500", "503", "504", "5XX"],
            retry_config=retry_config,
        )

        data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.BookingCancelTradeAllocationResponse(
                cancel_trade_allocation_response=utils.unmarshal_json(
                    http_res.text, Optional[components.CancelTradeAllocationResponse]
                ),
                http_meta=components.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(
            http_res, ["400", "403", "404", "500", "503", "504"], "application/json"
        ):
            data = utils.unmarshal_json(http_res.text, errors.StatusData)
            raise errors.Status(data=data)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )
        if utils.match_response(http_res, "default", "application/json"):
            return operations.BookingCancelTradeAllocationResponse(
                status=utils.unmarshal_json(http_res.text, Optional[components.Status]),
                http_meta=components.HTTPMetadata(request=req, response=http_res),
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    async def cancel_trade_allocation_async(
        self,
        *,
        account_id: str,
        trade_allocation_id: str,
        cancel_trade_allocation_request_create: Union[
            components.CancelTradeAllocationRequestCreate,
            components.CancelTradeAllocationRequestCreateTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.BookingCancelTradeAllocationResponse:
        r"""Cancel Trade Allocation

        Cancel a trade allocation using the original trade_allocation_id.

        Upon successful submission, returns an empty response. CancelTradeAllocation will either cancel everything, or nothing at all if a failure occurs.

        :param account_id: The account id.
        :param trade_allocation_id: The tradeAllocation id.
        :param cancel_trade_allocation_request_create:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = operations.BookingCancelTradeAllocationRequest(
            account_id=account_id,
            trade_allocation_id=trade_allocation_id,
            cancel_trade_allocation_request_create=utils.get_pydantic_model(
                cancel_trade_allocation_request_create,
                components.CancelTradeAllocationRequestCreate,
            ),
        )

        req = self.build_request_async(
            method="POST",
            path="/booking/v1/accounts/{account_id}/tradeAllocations/{tradeAllocation_id}:cancel",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.cancel_trade_allocation_request_create,
                False,
                False,
                "json",
                components.CancelTradeAllocationRequestCreate,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="Booking_CancelTradeAllocation",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "403", "404", "4XX", "500", "503", "504", "5XX"],
            retry_config=retry_config,
        )

        data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.BookingCancelTradeAllocationResponse(
                cancel_trade_allocation_response=utils.unmarshal_json(
                    http_res.text, Optional[components.CancelTradeAllocationResponse]
                ),
                http_meta=components.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(
            http_res, ["400", "403", "404", "500", "503", "504"], "application/json"
        ):
            data = utils.unmarshal_json(http_res.text, errors.StatusData)
            raise errors.Status(data=data)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )
        if utils.match_response(http_res, "default", "application/json"):
            return operations.BookingCancelTradeAllocationResponse(
                status=utils.unmarshal_json(http_res.text, Optional[components.Status]),
                http_meta=components.HTTPMetadata(request=req, response=http_res),
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    def rebook_trade_allocation(
        self,
        *,
        account_id: str,
        trade_allocation_id: str,
        rebook_trade_allocation_request_create: Union[
            components.RebookTradeAllocationRequestCreate,
            components.RebookTradeAllocationRequestCreateTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.BookingRebookTradeAllocationResponse:
        r"""Rebook Trade Allocation

        Rebook a trade allocation by the original trade_allocation_id. The allocation is rebooked by canceling the original allocation and creating a new one with the provided details.

        Upon successful submission, returns both the original and new allocation, as separate resources.

        :param account_id: The account id.
        :param trade_allocation_id: The tradeAllocation id.
        :param rebook_trade_allocation_request_create:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = operations.BookingRebookTradeAllocationRequest(
            account_id=account_id,
            trade_allocation_id=trade_allocation_id,
            rebook_trade_allocation_request_create=utils.get_pydantic_model(
                rebook_trade_allocation_request_create,
                components.RebookTradeAllocationRequestCreate,
            ),
        )

        req = self.build_request(
            method="POST",
            path="/booking/v1/accounts/{account_id}/tradeAllocations/{tradeAllocation_id}:rebook",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.rebook_trade_allocation_request_create,
                False,
                False,
                "json",
                components.RebookTradeAllocationRequestCreate,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = self.do_request(
            hook_ctx=HookContext(
                operation_id="Booking_RebookTradeAllocation",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "403", "404", "4XX", "500", "503", "504", "5XX"],
            retry_config=retry_config,
        )

        data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.BookingRebookTradeAllocationResponse(
                rebook_trade_allocation_response=utils.unmarshal_json(
                    http_res.text, Optional[components.RebookTradeAllocationResponse]
                ),
                http_meta=components.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(
            http_res, ["400", "403", "404", "500", "503", "504"], "application/json"
        ):
            data = utils.unmarshal_json(http_res.text, errors.StatusData)
            raise errors.Status(data=data)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )
        if utils.match_response(http_res, "default", "application/json"):
            return operations.BookingRebookTradeAllocationResponse(
                status=utils.unmarshal_json(http_res.text, Optional[components.Status]),
                http_meta=components.HTTPMetadata(request=req, response=http_res),
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )

    async def rebook_trade_allocation_async(
        self,
        *,
        account_id: str,
        trade_allocation_id: str,
        rebook_trade_allocation_request_create: Union[
            components.RebookTradeAllocationRequestCreate,
            components.RebookTradeAllocationRequestCreateTypedDict,
        ],
        retries: OptionalNullable[utils.RetryConfig] = UNSET,
        server_url: Optional[str] = None,
        timeout_ms: Optional[int] = None,
    ) -> operations.BookingRebookTradeAllocationResponse:
        r"""Rebook Trade Allocation

        Rebook a trade allocation by the original trade_allocation_id. The allocation is rebooked by canceling the original allocation and creating a new one with the provided details.

        Upon successful submission, returns both the original and new allocation, as separate resources.

        :param account_id: The account id.
        :param trade_allocation_id: The tradeAllocation id.
        :param rebook_trade_allocation_request_create:
        :param retries: Override the default retry configuration for this method
        :param server_url: Override the default server URL for this method
        :param timeout_ms: Override the default request timeout configuration for this method in milliseconds
        """
        base_url = None
        url_variables = None
        if timeout_ms is None:
            timeout_ms = self.sdk_configuration.timeout_ms

        if server_url is not None:
            base_url = server_url

        request = operations.BookingRebookTradeAllocationRequest(
            account_id=account_id,
            trade_allocation_id=trade_allocation_id,
            rebook_trade_allocation_request_create=utils.get_pydantic_model(
                rebook_trade_allocation_request_create,
                components.RebookTradeAllocationRequestCreate,
            ),
        )

        req = self.build_request_async(
            method="POST",
            path="/booking/v1/accounts/{account_id}/tradeAllocations/{tradeAllocation_id}:rebook",
            base_url=base_url,
            url_variables=url_variables,
            request=request,
            request_body_required=True,
            request_has_path_params=True,
            request_has_query_params=True,
            user_agent_header="user-agent",
            accept_header_value="application/json",
            security=self.sdk_configuration.security,
            get_serialized_body=lambda: utils.serialize_request_body(
                request.rebook_trade_allocation_request_create,
                False,
                False,
                "json",
                components.RebookTradeAllocationRequestCreate,
            ),
            timeout_ms=timeout_ms,
        )

        if retries == UNSET:
            if self.sdk_configuration.retry_config is not UNSET:
                retries = self.sdk_configuration.retry_config

        retry_config = None
        if isinstance(retries, utils.RetryConfig):
            retry_config = (retries, ["429", "500", "502", "503", "504"])

        http_res = await self.do_request_async(
            hook_ctx=HookContext(
                operation_id="Booking_RebookTradeAllocation",
                oauth2_scopes=[],
                security_source=self.sdk_configuration.security,
            ),
            request=req,
            error_status_codes=["400", "403", "404", "4XX", "500", "503", "504", "5XX"],
            retry_config=retry_config,
        )

        data: Any = None
        if utils.match_response(http_res, "200", "application/json"):
            return operations.BookingRebookTradeAllocationResponse(
                rebook_trade_allocation_response=utils.unmarshal_json(
                    http_res.text, Optional[components.RebookTradeAllocationResponse]
                ),
                http_meta=components.HTTPMetadata(request=req, response=http_res),
            )
        if utils.match_response(
            http_res, ["400", "403", "404", "500", "503", "504"], "application/json"
        ):
            data = utils.unmarshal_json(http_res.text, errors.StatusData)
            raise errors.Status(data=data)
        if utils.match_response(http_res, ["4XX", "5XX"], "*"):
            raise errors.SDKError(
                "API error occurred", http_res.status_code, http_res.text, http_res
            )
        if utils.match_response(http_res, "default", "application/json"):
            return operations.BookingRebookTradeAllocationResponse(
                status=utils.unmarshal_json(http_res.text, Optional[components.Status]),
                http_meta=components.HTTPMetadata(request=req, response=http_res),
            )

        content_type = http_res.headers.get("Content-Type")
        raise errors.SDKError(
            f"Unexpected response received (code: {http_res.status_code}, type: {content_type})",
            http_res.status_code,
            http_res.text,
            http_res,
        )
